/*
 * periph.c : UZK main processes
 *   by uratan! 2008.9.18
 */
#include "common.h"
#include "LPC214x.h"
#include "dscr.h.S"
#include "usb.h"
#include "usbhw.h"

/*
 * my local debug flag
 */
#define UDEBUG 1
#define rPRINTF  if(UDEBUG)rprintf

/*
 * defines for HID device
 */
#define UR_GET_REPORT			0x01
#define UR_SET_REPORT			0x09
#define UR_GET_IDLE				0x02
#define UR_SET_IDLE				0x0a
#define     IDLE_TIME_RATIO        4	/* [msec/1] */
#define UR_GET_PROTOCOL			0x03
#define UR_SET_PROTOCOL			0x0b

/*
 * my device's pipe infos (sync with each descriptors in dscr.a51 !)
 */
#if MOUSE
# define IF_HID1     0		/* interface # of HID1 (Keyboard) is 0 */
# define IF_HID2     1		/* interface # of HID2 (Control) is 1 */
# define IF_HID3     2		/* interface # of HID3 (Mouse)  is 2 */
# define N_IFs       3		/* total number of interfaces */
#else /* MOUSE */
# define IF_HID1     0		/* interface # of HID1 (Keyboard) is 0 */
# define IF_HID2     1		/* interface # of HID2 (Control) is 1 */
# define N_IFs       2		/* total number of interfaces */
#endif /* MOUSE */

/* as keyboard */
#define HID1_IN_DATA_LEN   8	/* keycodes --> HOST */
#define HID1_OUT_DATA_LEN  1	/* LED status <-- HOST */

/* as vendor device */
#define HID2_IN_DATA_LEN   2	/* LED status/control_status --> HOST */
#define HID2_OUT_DATA_LEN  9	/* keycodes/mouse/control_status <-- HOST */

/* as mouse */
#define HID3_IN_DATA_LEN   4	/* mouse --> HOST */


/*
 * functions
 */
/* in myself */
void TD_Init(void);
void TD_Poll(void);
int TD_Suspend(void);
void TD_Resume(void);
int DR_SetupCommand(void);
int DR_GetDescriptor(void);
void xbzero(u_char *dst, u_long len);
void xmemcpy(u_char *dst, u_char *src, u_long len);
void filter_mouse_data(u_char *dst, u_long len);
void isr_Reset_Event(u_long unused);
void isr_Suspend_Event(u_long unused);
void isr_Resume_Event(u_long unused);
void isr_EP1_in(u_long x);
void isr_EP2_in(u_long x);
void isr_EP3_in(u_long x);
/* jump tables */
const INTR_subFUNCs USB_Jump_Table[N_JT];
const INTR_subFUNCs USB_Jump_Table_EP[LPC_N_EP];
/* in usbhw.c */
extern u_long uhw_WriteEP(u_char EPNum, u_char *pData, u_long cnt);
extern void uhw_arm_WriteEP(u_char EPNum);
//extern int uhw_ReadEP(u_char EPNum, u_char *pData, u_long DataSize);
/* in usbcore.c */
//extern void ucore_ResetCore(void);
extern void ucore_start_DataIn(u_char *buff, u_long len);
extern void ucore_start_DataOut(u_char *buff, u_long len, u_long bufsize);
extern void ucore_wait_DataOut_finished(void);
extern void ucore_start_StatusIn(void);
extern void ucore_STALL_EP0(void);
/* in uart1.c */
extern int uart1_init(u_long baudrate);
extern int peek_recv(void);
extern void set_to_send(u_char c);
/* in timer0.c */
void init_timer0(void);

/* in dscr.S */
extern u_long *HIDDscr1, *ReportDscr1, *ReportDscr1Len;
extern u_long *HIDDscr2, *ReportDscr2, *ReportDscr2Len;
#if MOUSE
extern u_long *HIDDscr3, *ReportDscr3, *ReportDscr3Len;
#endif /* MOUSE */
/* in usbcore.c */
extern u_char SETUPDAT[8];
extern u_char EP0BUF[DSCR_MPSIZE_EP0];
/* in fw.c */
//extern BOOL	GotSUD;			// Received setup data flag
//extern BOOL	Sleep;
//extern BOOL	Rwuen;
/* in timer0.c */
extern volatile int timer0_tick;

/* for myself */
/*
 * device status
 */
extern u_char USB_Configuration;	/* 0 or configuration number */
//BYTE Configuration;		// Current configuration
//BYTE AlternateSetting;	// Alternate settings

/*
 * macros for accessing Nth endpoint registers
 */
//#define INxBUF(x)   (IN1BUF - 128*(x))
//#define INxBC(x)    EPIO[((x)+1)].bytes
const u_char EP_IN[N_IFs] = {
	DSCR_ADDR_EP1,	/* KB */
	DSCR_ADDR_EP2,	/* CTRL */
#if MOUSE
	DSCR_ADDR_EP3,	/* MS */
#endif /* MOUSE */
};

/*
 * means "endpoint is already armed, waiting for IN requeset from HOST" flag
 *   [caution] set in TD_Poll(), reset by interrupt handler
 */
	/*
	 * uGh|Cg͂f[^]NꂿAƂ
	 *   zXgǂ݂ɗ̂҂ȂvtOB
	 * ǋNĂ܂~߂Ȃ̂ŁAobt@̓e
	 * r䂷邽߂̃tOB荞݃nhŃNA邽߂
	 * oCĝ܂ܗpӂ܂B
	 *   (rbgɃpbNƁu[hfBt@CCgvɂȂ
	 *    ̕ϐ̃ANZX̂ɔr䂪KvɂȂĂ܂)
	 */
u_char INxBUF_is_busy[N_IFs];

/*
 * interface/endpoint status
 */
u_char Protocol[N_IFs];		/* (nothing is influenced by this value) */
u_char Idle_Rate[N_IFs];
u_short idle_timer[N_IFs];
u_char work_INxBUF[N_IFs][DSCR_MPSIZE_EP123];
u_char INxBUF_backup[N_IFs][DSCR_MPSIZE_EP123];

#define HT_NO_TRIGGER          0
#define HT_WORK_INxBUF_READY   1	/* working INxBUF is ready to send */
#define HT_IDLE_TIME_ELAPSED   2	/* idle time elapsed, enable tansfer */
u_char HIDx_trigger[N_IFs];

/*
 * consts for accessing Nth endpoint data length
 */
const u_char HIDx_in_data_len[N_IFs] = {
	HID1_IN_DATA_LEN,
	HID2_IN_DATA_LEN,
#if MOUSE
	HID3_IN_DATA_LEN,
#endif /* MOUSE */
};

/*
 * serial1 comminucation protocol
 *  reset
 *     +----+
 *     |0xff|
 *     +----+
 *  send LED status (payload is 1 byte)
 *     +----+ +----+
 *     |'L' | |    |
 *     +----+ +----+
 *  send keycodes (payload is 8 bytes)
 *     +----+ +----+----+----+----+----+----+----+----+----+
 *     |'K' | |    |    |    |    |    |    |    |    |    |
 *     +----+ +----+----+----+----+----+----+----+----+----+
 *  send mouse action (payload is 4 bytes)
 *     +----+ +----+----+----+----+
 *     |'M' | |    |    |    |    |
 *     +----+ +----+----+----+----+
 *  send control status (payload is 1 byte)
 *     +----+ +----+
 *     |'C' | |    |
 *     +----+ +----+
 *     |<------------------------------------------------->|
 *        all contents including header is passed for/by HID2
 *            |<------------------------------------------>|
 *               only payload is passed for/by HID1,HID3
 */
	/*
	 * L[{[hE}EXƂĔF HID1/HID3 ͓Rwb_[Ȃ
	 * Reĉ݂肵܂Ap HID2 ʂƂ
	 * wb_[łB܂A
	 * 'K', 'M', 'C' ͐e PC  HID2 ɏނƂ 9oCgŒ蒷
	 * ł uart1 ɗƂɏk߂Ă܂B
	 */
#define HEAD_RESET  0xff	/* you can not pass this as contents !!! */
#define HEAD_LEDS   'L'
#define HEAD_CTRL   'C'		/* misc control for future use */
#define HEAD_KEYS   'K'
#define HEAD_MOUS   'M'
/*
 * remap signed value for excluding HEAD_RESET(0xff)
 *
 *   orginal value:    0x81, ..., 0xfe, 0xff, 0x00, 0x01, 0x02, ..., 0x7f
 *                    (-127, ...,   -2,   -1,    0,    1,    2, ...,  127)
 *   ---------------------------------------------------------------------
 *   converted value:  0xfe, ..., 0x81, 0x80, 0x7f, 0x7e, 0x7d, ..., 0x00
 *
 *       CAUTION : you can still NOT send -128 (0x80) !
 */
#define MOUSE_DATA_FILTER   0x7f	/* make xor with 0x7f, or (127-x) */
	/*
	 * MɁuv HEAD_RESET 𑗂肽̂ŃRec
	 * 0xff ܂ł͂ȂȂB}EXf[^͂\̂
	 * ̂悤ɕϊĂƂ肵܂B
	 */

/*
 * serial1 comminucation receiver states
 */
#define RECV_for_HID1       0	/* ==IF_HID1 */
#define RECV_for_HID2       1	/* ==IF_HID2 */
#define RECV_for_HID3       2	/* ==IF_HID3 */
#define RECV_NULL_or_IDLE   3	/* ==N_IFs */
u_char recv_target;				/* I have to handle HEAD_MOUS when !MOUSE */
u_char recv_ptr;

/*
 * monitor LED
 */
//#define MON_LED_OFF    OUTC |= 0x10
//#define MON_LED_ON     OUTC &= ~0x10
u_char mon_led_timer;


/*
 *
 */
void TD_Init(void) 				// Called once at startup
{
	int i;

//	/*
//	 * Enable endpoints
//	 */
//#if MOUSE
//	IN07VAL |= bmEP1 + bmEP2 + bmEP3;
//#else /* MOUSE */
//	IN07VAL |= bmEP1 + bmEP2;
//#endif /* MOUSE */
//
//	/*
//	 * Enable interrupts for endpoints
//	 */
//#if MOUSE
//	IN07IEN |= bmEP1 + bmEP2 + bmEP3;
//#else /* MOUSE */
//	IN07IEN |= bmEP1 + bmEP2;
//#endif /* MOUSE */

//	Rwuen = (WORD)&DscrRWU;		// copy from descriptor (dscr.a51)

	/*
	 * here from by uratan!
	 */
//#if UDEBUG
//	init_uart0();	/* uart0 for debug */
//#endif /* UDEBUG */
//	init_uart1();	/* uart1 for comminication with the other */

//	OEC |= bmBIT4;		// only PC4 is output now (LED)
//	CPUCS &= ~bmCLK24OE;	// disable output of 24MHz

//	PUTstring("\n/*\n * start EZ-USB\n */\n");

//	PUTstring("Chip Revision:");
//	PUThex(GET_CHIP_REV(), 1);
//	PUTstring(" (0=A, 2=C, 3=D, 4=E)\n");

//	PUThex(IE, 2);
//	PUTstring(": EA\n");

	/*
	 * initial device
	 */
//	Configuration = 0;
//	AlternateSetting = 0;

	/*
	 * initial each interface
	 */
	for(i=0; i<N_IFs; i++) {
//		xbzero(INxBUF(i), HIDx_in_data_len[i]);
// not realized at here !!!
		xbzero(INxBUF_backup[i], HIDx_in_data_len[i]);
		INxBUF_is_busy[i] = FALSE;

		Protocol[i] = 1;		/* boot device */

		idle_timer[i] = 0;
		HIDx_trigger[i] = HT_NO_TRIGGER;
	}

	/*
	 * HID spec sheet says that
	 *   recommanded default idle rate is
	 *     500ms for keyboard, 0ms for mouse.
	 */
	Idle_Rate[IF_HID1] = 500/*[msec]*/ / IDLE_TIME_RATIO;
	Idle_Rate[IF_HID2] =   0/*[msec]*/ / IDLE_TIME_RATIO;
#if MOUSE
	Idle_Rate[IF_HID3] =   0/*[msec]*/ / IDLE_TIME_RATIO;
#endif /* MOUSE */

	/*
	 * initial others
	 */
	recv_target = RECV_NULL_or_IDLE;		/* reset state */

	mon_led_timer = 0;

//	init_timer0();

	return;
}

/*
 * Called repeatedly while the device is idle
 */
void TD_Poll(void)
{
	int d, i;

	/*
	 * this function takes only 60usec when idle.
	 */

#if UDEBUG
	d = cm_keyin();
	if(d != -1) {
		cm_putchar(d);
	}
#endif /* UDEBUG */

	/*
	 * is something coming from another side of UZK ?
	 */
	d = peek_recv();
	if(d != -1) {
//		PUThex(d, 2);
//		PUTchar('\n');

		MON_LED_OFF;	/* turn off LED cause something act */

		if(d == HEAD_RESET) {
			recv_target = RECV_NULL_or_IDLE;		/* reset state */
		} else {
			switch(recv_target) {
			case RECV_NULL_or_IDLE:
					if(d == HEAD_KEYS)
						recv_target = RECV_for_HID1;	/* change state */
					else if((d == HEAD_LEDS) || (d == HEAD_CTRL))
						recv_target = RECV_for_HID2;	/* change state */
					else if(d == HEAD_MOUS)
						recv_target = RECV_for_HID3;	/* change state */
					else
						;							/* do not change state */
					recv_ptr = 0;
					break;

			case RECV_for_HID3:
#if !MOUSE
					/*
					 * I don't have endpoint for mouse, so feed them only
					 */
						/*
						 * ΂ɂȂĂ EZ-USB Ɠt@[g
						 * Ƃ͌ȂȂB
						 */
					recv_ptr++;
					if(recv_ptr == HID3_IN_DATA_LEN)
						recv_target = RECV_NULL_or_IDLE;	/* reset state */
					break;
#endif /* !MOUSE */
			case RECV_for_HID1:
			case RECV_for_HID2:
					/*
					 * threw previous work_INxBUF value away
					 * wheather they are still valid ot not.
					 */
					HIDx_trigger[recv_target] &= ~HT_WORK_INxBUF_READY;
						/*
						 * work_INxBUF Ƀf[^܂ĂĂ
						 * ̂ĂĂ܂܂B
						 */

					work_INxBUF[recv_target][recv_ptr++] = d;
					if(recv_ptr == HIDx_in_data_len[recv_target]) {
						HIDx_trigger[recv_target] |= HT_WORK_INxBUF_READY;
						if(recv_target==RECV_for_HID3) {
							filter_mouse_data(&work_INxBUF[RECV_for_HID3][0], HID3_IN_DATA_LEN);
						}
						recv_target = RECV_NULL_or_IDLE;	/* reset state */
					}
					break;

			default:
					recv_target = RECV_NULL_or_IDLE;	/* reset state */
					break;
			}
		}
	}

	/*
	 * measure report interval time
	 */
	if(timer0_tick) {
		/*
		 * clear the flag
		 */
		timer0_tick = FALSE;

		/*
		 * turn on monitor every interval time
		 */
		if(++mon_led_timer == 20/*[msec]*/) {
			mon_led_timer = 0;	/* reset timer */
			MON_LED_ON;			/* turn on LED here */
		}

		if(USB_Configuration) {
			idle_timer[IF_HID1]++;
//			idle_timer[IF_HID2]++;	/* <== HID2 idle timer is disabled here */
#if MOUSE
			idle_timer[IF_HID3]++;
#endif /* MOUSE */
		}
	}

	/*
	 * loop for all endpoints
	 */
	for(i=0; i<N_IFs; i++) {
		/*
		 * is interval time elapsed ?
		 */
		if(Idle_Rate[i] == 0) {
			idle_timer[i] = 0;
				/*
				 * it seems that HID drivers on recently OS
				 * always set 0 to Idle_Rate.
				 */
// except Mac OS X 10.4 (24msec)
					/*
					 * ݂͈͂ OS ͕K "SET_IDLE 0" Ă܂B
					 */
		} else {
			if(idle_timer[i] >= Idle_Rate[i] * IDLE_TIME_RATIO) {
				HIDx_trigger[i] |= HT_IDLE_TIME_ELAPSED;
				idle_timer[i] = 0;
					/*
					 * I want to disable timer trigger on HID2 in any case.
					 * see 26 lines above.
					 */
						/*
						 * HID2 (UZK CTRL) ɁAԐ؂ł̃|[g
						 * Ȃ߂ɁAŃ^C}[JEgĂȂ
						 */
			}
		}
	}

	/*
	 * loop for all endpoints
	 */
	for(i=0; i<N_IFs; i++) {
		/*
		 * arm the endpoint if condition met
		 */
		if(INxBUF_is_busy[i]) {
			/*
			 * the endpoint is already armed, but IN request does not
			 * come yet. do NOT touch INxBUF because I don't know
			 * when IN request will come.
			 */
			;	/* do nothing now, keep state and try in next TD_Poll() loop */
					/*
					 * zXgf[^ǂ݂ɗ邩Ȃ
					 *  INxBUF ̓eύXłȂBύXĂ
					 * r̃f[^ǂ܂邩Ȃ
					 */
		} else {
			/*
			 * when new data arrived, copy contents from work_INxBUF.
			 *    (HT_WORK_INxBUF_READY is prior to HT_IDLE_TIME_ELAPSED)
			 */
			if(HIDx_trigger[i] & HT_WORK_INxBUF_READY) {
				/*
				 * disable interrupt while updating if any interrupt
				 * handler will access INxBUF.
				 * now, ISR_Ep3in() modifies IN3BUF but
				 * INxBUF_is_busy[IF_HID3] achieve the exclusive control.
				 */
//				EA = 0;		/* exclusive control against interrupt */
		// if(configured)
//				xmemcpy(INxBUF(i), work_INxBUF[i], HIDx_in_data_len[i]);
				xmemcpy(INxBUF_backup[i], work_INxBUF[i], HIDx_in_data_len[i]);
				uhw_WriteEP(EP_IN[i], work_INxBUF[i], HIDx_in_data_len[i]);
//				EA = 1;
					/*
					 * ISR_Ep3in() ł IN3BUF ɃANZXĂ̂
					 * INxBUF ̃ANZXɔr䂪KvƎvA
					 * ϐ INxBUF_is_busy[] ł܂r䂳Ă
					 * 悤Ȃ̂Ŋ荞݂֎~Ă܂ł̔r͂ȂB
					 *                                          (...͂)
					 */
			}
			else if(HIDx_trigger[i] & HT_IDLE_TIME_ELAPSED) {
				uhw_WriteEP(EP_IN[i], INxBUF_backup[i], HIDx_in_data_len[i]);
			}

			/*
			 * when timer trigger, send remained data(*) in INxBUF again.
			 *   there are only *state* data in Keycodes and LED status.
			 *   so I can report same data over and over again.
			 *   mouse has both *state* data (button) and *differential*
			 *   data (pointer). *differential* data once sent are
			 *   cleared in ISR_Ep3in()
			 */
				/*
				 * (gpł͂܂Ȃ񂾂)Ԑ؂ɂ郌|[gł́A
				 * Ȃ̂őÕf[^̂܂܍đB
				 * L[{[h̏ꍇɂ́uOƓL[vɂȂ̂ŖȂA
				 * }EX̏ꍇ́uOƓړʁvɂȂĂ܂
				 * 낵ȂBȂ̂Ŏ ISR_Ep3in() ł͑Mɍ
				 * f[^NAĂB
				 */
			if(HIDx_trigger[i]) {
				HIDx_trigger[i] = HT_NO_TRIGGER;

				if(USB_Configuration) {
					INxBUF_is_busy[i] = TRUE;
//					INxBC(i) = HIDx_in_data_len[i];	/* contents ready now */
					uhw_arm_WriteEP(EP_IN[i]);
					idle_timer[i] = 0;		/* measure idle time from here */
				}
			}
		}
	}
}

/*
 * Called before the device goes into suspend mode
 */
int TD_Suspend(void)
{
	MON_LED_OFF;	/* turn off LED */

	return(DONE);
}

/*
 * Called after the device resumes
 */
void TD_Resume(void)
{
	return;
}


/*
 * HID Requset Support Requirements
 *
 *                     Get      Set      Get      Set      Get      Set
 *   Device Type       Report   Report   Idle     Idle     Protocol Protocol
 * -------------------------------------------------------------------------
 *  Boot Mouse         Required Optional Optional Optional Required Required
 *  Non-Boot Mouse     Required Optional Optional Optional Optional Optional
 *  Boot Keyboard      Required Optional Required Required Required Required
 *  Non-Boot Keyboard  Required Optional Required Required Optional Optional
 *  Other Device       Required Optional Optional Optional Optional Optional
 */
	/*
	 * ܂ HID ̎dl悭ǂłꂽ܂B
	 * ϐ Protocol[] ̓e͂͂Ă܂
	 * œς͂Ă܂B
	 */
/*
 * Device Request hook function
 */
int DR_SetupCommand(void)
{
	u_char target_hid;
	BOOL is_hid1;
#if MOUSE
	BOOL is_hid3;
#endif /* MOUSE */
	u_char ans_len, req_len, i;

	MON_LED_OFF;	/* turn off LED cause something act */

	/*
	 * feed class request only
	 */
	if((SETUPDAT[0] & UT_TYPE_MASK) != UT_CLASS)
		return(YET);

	/*
	 * which interface is target ?
	 */
	if((SETUPDAT[5] != 0x00) || (SETUPDAT[4] >= N_IFs))
		goto exit_with_stall;

	target_hid = SETUPDAT[4];
	is_hid1 = (SETUPDAT[4] == IF_HID1);
#if MOUSE
	is_hid3 = (SETUPDAT[4] == IF_HID3);
#endif /* MOUSE */

	/*
	 * I shall not handle so long data
	 */
	if(SETUPDAT[7] != 0x00)
		goto exit_with_stall;

	req_len = SETUPDAT[6];
	ans_len = 0;			/* no need to answer */

	switch(SETUPDAT[1]) {
	case UR_GET_REPORT:			/* 0x01 mandatory */
			rPRINTF("GET_REPO\r\n");
			ans_len = HIDx_in_data_len[target_hid];
//			xmemcpy(EP0BUF, INxBUF(target_hid), ans_len);
			xmemcpy(EP0BUF, INxBUF_backup[target_hid], ans_len);
//			uhw_ReadEP(EP_IN[target_hid], EP0BUF, ans_len);
//			// can I read in endpoint memory ?
//			// if(configured)
			break;

	case UR_GET_IDLE:			/* 0x02 optional */
			rPRINTF("GET_IDLE\r\n");
			EP0BUF[0] = Idle_Rate[target_hid];
			ans_len = 1;
			break;

	case UR_SET_IDLE:			/* 0x0a optional */
			rPRINTF("SET_IDLE - %02x\r\n", SETUPDAT[3]);
			/*
			 * ignore if shoter than Polling interval
			 * in endpoint descriptor (10msec)
			 */
			if(SETUPDAT[3] == 0)
				Idle_Rate[target_hid] = 0;	/* indefinite */
			else if(SETUPDAT[3] < 3)
				Idle_Rate[target_hid] = 3;	/* minimum */
			else
				Idle_Rate[target_hid] = SETUPDAT[3];

			idle_timer[target_hid] = 0;
			break;

	case UR_GET_PROTOCOL:		/* 0x03 mandatory if boot dev */
			rPRINTF("GET_PROT\r\n");
			EP0BUF[0] = Protocol[target_hid];
			ans_len = 1;
			break;

	case UR_SET_PROTOCOL:		/* 0x0b mandatory if boot dev */
			rPRINTF("SET_PROT - %02x\r\n", SETUPDAT[2]);
			Protocol[target_hid] = SETUPDAT[2];
			break;

	case UR_SET_REPORT:			/* 0x09 optional */
			rPRINTF("SET_REPO -");
#if MOUSE
			if(is_hid3)
				goto exit_with_stall;
#endif /* MOUSE */

			if(is_hid1)
				i = HID1_OUT_DATA_LEN;
			else
				i = HID2_OUT_DATA_LEN;

			if(req_len != i)
				goto exit_with_stall;

			/*
			 * Arm endpoint - do it here to clear (after sud avail)
			 * Clear bytecount to allow new data in; also stops NAKing
			 */
//			OUT0BC = 0;
//			while(EP0CS & 0x08)
//				;

			ucore_start_DataOut(EP0BUF, i, 64);
			ucore_wait_DataOut_finished();

#if UDEBUG
			for(i=0; i<req_len; i++)
				rprintf(" %02x", EP0BUF[i]);
			rprintf("\r\n");
#endif /* UDEBUG */

//			if(OUT0BC != req_len)
//				goto exit_with_stall;
			/*
			 * rewind communication every time
			 * cause I don't know partner's state.
			 * he may NOT be in `RECV_NULL_or_IDLE`
			 * by power-down or suspend of my side.
			 */
				/*
				 * uart1 ̌ɂ鑊ǂȏԂɂ邩
				 * Ȃ̂ŁAMƂɏ߂𑗂邱Ƃɂ
				 */
			set_to_send(HEAD_RESET);
				/*
				 * because here is NOT interrupt layer, I can wait.
				 * so set_to_send() waits when uart send buff is full.
				 * at the result, transaction with host shall be late.
				 */
					/*
					 * ͂ Setup Command ̏́A荞ݏł
					 * Ȃ|[ÖꕔȂ̂ŁAԂĂ
					 * ̂łBȂ̂łłɑMobt@ς
					 *  set_to_send() ֐̓obt@󂭂܂ő҂悤
					 * ܂BN҂̂ƂƁASET_REPORT 
					 * s PC ҂܂B
					 */

			if(is_hid1) {
				set_to_send(HEAD_LEDS);	/* this is for uart1 comm */
				set_to_send(HEAD_LEDS);	/* this is for return data */
			} else {
				/*
				 * data from host have uart1 comminication header
				 * HEAD_KEYS/HEAD_MOUS/HEAD_CTRL on the top
				 */
				if(EP0BUF[0] == HEAD_KEYS)
					;
				else if(EP0BUF[0] == HEAD_MOUS) {
					req_len = 1+ HID3_IN_DATA_LEN;	/* reduce uart1 data size */
					filter_mouse_data(&EP0BUF[1], HID3_IN_DATA_LEN);
				} else if(EP0BUF[0] == HEAD_CTRL) {
					set_to_send(HEAD_CTRL);	/* this is for uart1 comm */
					req_len = HID2_IN_DATA_LEN;		/* reduce uart1 data size */
				} else
					goto exit_with_stall;
			}

			for(i=0; i<req_len; i++)
				set_to_send(EP0BUF[i]);
			break;

	default:
			goto exit_with_stall;
	}

	if(ans_len != 0) {
//		if(ans_len > req_len)
//			ans_len = req_len;
		ucore_start_DataIn(EP0BUF, ans_len);
	} else {
		ucore_start_StatusIn();
	}

	return(DONE);

	/*
	 * ---------------------------------
	 *    entry for return with error
	 * ---------------------------------
	 */
exit_with_stall:

	ucore_STALL_EP0();
//	cpipe_count = 0;

	return(DONE);
}

/*
 * Get Descriptor Device Request hook function
 */
int DR_GetDescriptor(void)
{
	BOOL is_hid1;
#if MOUSE
	BOOL is_hid3;
#endif /* MOUSE */
	u_char *dscr;
	u_long len;

	if((SETUPDAT[5] != 0x00) || (SETUPDAT[4] >= N_IFs)) {
		return(YET);
	}

	is_hid1 = (SETUPDAT[4] == IF_HID1);
#if MOUSE
	is_hid3 = (SETUPDAT[4] == IF_HID3);
#endif /* MOUSE */

	if(SETUPDAT[3]==UDESC_HID/*0x21*/) {
		rPRINTF("UDESC_HID\r\n");

#if MOUSE
		if(is_hid3) {
//			SUDPTRH = MSB((WORD)HIDDscr3);
//			SUDPTRL = LSB((WORD)HIDDscr3);
			dscr = (u_char *)&HIDDscr3;
			len = dscr[/*bLength*/ 0];
		} else
#endif /* MOUSE */
		if(is_hid1) {
//			SUDPTRH = MSB((WORD)HIDDscr1);
//			SUDPTRL = LSB((WORD)HIDDscr1);
			dscr = (u_char *)&HIDDscr1;
			len = dscr[/*bLength*/ 0];
		} else {
//			SUDPTRH = MSB((WORD)HIDDscr2);
//			SUDPTRL = LSB((WORD)HIDDscr2);
			dscr = (u_char *)&HIDDscr2;
			len = dscr[/*bLength*/ 0];
		}

		ucore_start_DataIn(dscr, len);
		return(DONE);
	}

	if(SETUPDAT[3]==UDESC_REPORT/*0x22*/) {
		rPRINTF("UDESC_REPO\r\n");
//	} else if(SETUPDAT[3]==UDESC_PHYSICAL/*0x23*/) {
//		rPRINTF("PHY\r\n");
	} else {
		return(YET);
	}

#if MOUSE
	if(is_hid3) {
//		answer_for_control_in((u_char *)&ReportDscr3, &ReportDscr3Len);
		dscr = (u_char *)&ReportDscr3;
		len = (u_long)&ReportDscr3Len;
	} else
#endif /* MOUSE */
	if(is_hid1) {
//		answer_for_control_in((u_char *)&ReportDscr1, &ReportDscr1Len);
		dscr = (u_char *)&ReportDscr1;
		len = (u_long)&ReportDscr1Len;
	} else {
//		answer_for_control_in((u_char *)&ReportDscr2, &ReportDscr2Len);
		dscr = (u_char *)&ReportDscr2;
		len = (u_long)&ReportDscr2Len;
	}

	ucore_start_DataIn(dscr, len);
	return(DONE);
}

/*
 * function to clear memories
 */
void xbzero(u_char *dst, u_long len)
{
	while(len--) {
		*dst = 0x00;
		dst++;
	}

	return;
}

/*
 * function to copy memories
 */
void xmemcpy(u_char *dst, u_char *src, u_long len)
{
	while(len--) {
		*dst = *src;
		dst++;
		src++;
	}

	return;
}

/*
 * convert mouse data contents for uart1 comminication HEAD_RESET byte value
 */
void filter_mouse_data(u_char *dst, u_long len)
{
	while(len--) {
		*dst ^= MOUSE_DATA_FILTER;
		dst++;
	}

	return;
}


/*
 * USB Endpoint Events Callback Pointers
 */
const INTR_subFUNCs USB_Jump_Table[N_JT] = {
/*0:USB_JT_RESET  */	isr_Reset_Event,
/*1:USB_JT_SUSPEND*/	isr_Suspend_Event,
/*2:USB_JT_RESUME */	isr_Resume_Event,
/*3:USB_JT_POWER  */	NULL  /*isr_Power_Event*/,
/*4:USB_JT_SOF    */	NULL  /*isr_SOF_Event*/,
/*5:USB_JT_ERROR  */	NULL  /*isr_Error_Event*/,
};

/*
 * USB Reset Event Callback
 */
void isr_Reset_Event(u_long unused)
{
	int i;

rPRINTF("uRes\r\n");

	for(i=0; i<N_IFs; i++) {
		INxBUF_is_busy[i] = FALSE;
		idle_timer[i] = 0;
	}
//	ucore_ResetCore();
}

/*
 * USB Suspend Event Callback
 */
void isr_Suspend_Event(u_long unused)
{
rPRINTF("uSus\r\n");
}

/*
 * USB Resume Event Callback
 */
void isr_Resume_Event(u_long unused)
{
rPRINTF("uRsm\r\n");
}

/*
 * use these if needed
 */
//void isr_Power_Event(u_long vbus_on) {;}
//void isr_SOF_Event(u_long frame_num) {;}
//void isr_Error_Event(u_long err) {;}

/*
 * USB Endpoint Events Callback Pointers
 */
const INTR_subFUNCs USB_Jump_Table_EP[LPC_N_EP] = {
/*00: 0out*/	NULL,		/* control out endpoint, unused */
/*80: 0in*/		NULL,		/* control in endpoint, unused */
/*01: 1out*/	 NULL,		/* interrupt OUT */
/*81: 1in*/		 isr_EP1_in,		/* interrupt IN */	/*KB*/
/*02: 2out*/	 NULL,		/* bulk OUT */
/*82: 2in*/		 NULL,		/* bulk IN */
/*03: 3out*/	 NULL,		/* iso OUT */
/*83: 3in*/		 NULL,		/* iso IN */
/*04: 4out*/	 NULL,		/* interrupt OUT */
/*84: 4in*/		 isr_EP2_in,		/* interrupt IN */	/*CTRL*/
/*05: 5out*/	 NULL,		/* bulk OUT */
/*85: 5in*/		 NULL,		/* bulk IN */
/*06: 6out*/	 NULL,		/* iso OUT */
/*86: 6in*/		 NULL,		/* iso IN */
/*07: 7out*/	 NULL,		/* interrupt OUT */
/*87: 7in*/		 isr_EP3_in,		/* interrupt IN */	/*MS*/
/*08: 8out*/	 NULL,		/* bulk OUT */
/*88: 8in*/		 NULL,		/* bulk IN */
/*09: 9out*/	 NULL,		/* iso OUT */
/*89: 9in*/		 NULL,		/* iso IN */
/*0a:10out*/	 NULL,		/* interrupt OUT */
/*8a:10in*/		 NULL,		/* interrupt IN */
/*0b:11out*/	 NULL,		/* bulk OUT */
/*8b:11in*/		 NULL,		/* bulk IN */
/*0c:12out*/	 NULL,		/* iso OUT */
/*8c:12in*/		 NULL,		/* iso IN */
/*0d:13out*/	 NULL,		/* interrupt OUT */
/*8d:13in*/		 NULL,		/* interrupt IN */
/*0e:14out*/	 NULL,		/* bulk OUT */
/*8e:14in*/		 NULL,		/* bulk IN */
/*0f:15out*/	 NULL,		/* bulk OUT */
/*8f:15in*/		 NULL,		/* bulk IN */
};

///*
// * Setup Data Available Interrupt Handler
// */
//void ISR_Sudav(void) interrupt 0 using 2
//{
////	PUTstring("Sudav\n");
//
//	GotSUD = TRUE;				// Set flag
//
//	EZUSB_IRQ_CLEAR();
//	USBIRQ = bmSUDAV;			// Clear SUDAV IRQ
//}
//
///*
// * USB Reset Interrupt Handler
// */
//void ISR_Ures(void) interrupt 0 using 2
//{
//	BYTE i;
//
//	PUTstring("Ures\n");
//
//	GotSUD = FALSE;			// clear old status
//	Configuration = 0;		// clear Current configuration
//
//	for(i=0; i<N_IFs; i++) {
//		INxBUF_is_busy[i] = FALSE;
//		idle_timer[i] = 0;
//	}
//		/* others may be re-initialized here ... */
//
//	EZUSB_IRQ_CLEAR();
//	USBIRQ = bmURES;			// Clear URES IRQ
//}
//
///*
// * Global Suspend Interrupt Handler
// */
//void ISR_Susp(void) interrupt 0 using 2
//{
//	PUTstring("Susp\n");
//
//	Sleep = TRUE;
//
//	EZUSB_IRQ_CLEAR();
//	USBIRQ = bmSUSP;
//}
//
void isr_EP1_in(u_long x)
{
	rPRINTF("!1\r\n");

	/*
	 * this entry is called after IN request has success...
	 */
		/*
		 *  IN 荞݂́AIN NGXgăobt@
		 * ɂȂƂ(̃f[^pӂ邽߂)Ăяo܂B
		 */
	INxBUF_is_busy[IF_HID1] = FALSE;

//	EZUSB_IRQ_CLEAR();
//	IN07IRQ = bmEP1;
}

void isr_EP2_in(u_long x)
{
	rPRINTF("!2\r\n");
	/*
	 * this entry is called after IN request has success...
	 */
	INxBUF_is_busy[IF_HID2] = FALSE;

//	EZUSB_IRQ_CLEAR();
//	IN07IRQ = bmEP2;
}

void isr_EP3_in(u_long x)
{
	rPRINTF("!3\r\n");
	/*
	 * this entry is called after IN request has success...
	 */

#if MOUSE
	/*
	 * button state is kept, but *differential* data once sent are cleared.
	 */
		/*
		 * }EX̏ꍇ́uړʁṽf[^݂̂NAĂ܂B
		 *  SET_IDLE ɂ鎞Ԍoߌɍđꍇ̂߂łB
		 */
//	IN3BUF[1] = 0;	/* X */
//	IN3BUF[2] = 0;	/* Y */
//	IN3BUF[3] = 0;	/* WHEEL */
	INxBUF_backup[IF_HID3][1] = 0;	/* X */
	INxBUF_backup[IF_HID3][2] = 0;	/* Y */
	INxBUF_backup[IF_HID3][3] = 0;	/* WHEEL */
//		xbzero(INxBUF_backup[i], HIDx_in_data_len[i]);

	INxBUF_is_busy[IF_HID3] = FALSE;
#endif /* MOUSE */

//	EZUSB_IRQ_CLEAR();
//	IN07IRQ = bmEP3;
}
//
///*
// * dummy Interrupt Handler for out of use Interrupts
// */
//	/*
//	 * Ȃɂ̂Ƃɖ\̂ȂȂ̂ŁAŖ[vɗƂ܂B
//	 */
//void ISR_dummy(void) interrupt 0 using 2
//{
//	PUTstring("ISR_dummy!");
//
//	while(1)
//		;		/* freeze here */
//}
