/*
 * usbhregs.h : constants about LPC2148 USB registers
 *   by uratan! 2008.9.22
 */
#ifndef __USBREG_H__
#define __USBREG_H__

/*
 * DEV_INT_EN, DEV_INT_STAT, DEV_INT_CLR: Device Interrupt Bit Definitions
 */
#define UDI_FRAME_INT       0x00000001
#define UDI_EP_FAST_INT     0x00000002
#define UDI_EP_SLOW_INT     0x00000004
#define UDI_DEV_STAT_INT    0x00000008
#define UDI_CCEMTY_INT      0x00000010
#define UDI_CDFULL_INT      0x00000020
#define UDI_RxENDPKT_INT    0x00000040
#define UDI_TxENDPKT_INT    0x00000080
#define UDI_EP_RLZED_INT    0x00000100
#define UDI_ERR_INT         0x00000200

/*
 * RX_PLENGTH: Rx & Tx Packet Length Definitions
 */
#define PKT_LENGTH_MASK     0x000003FF
#define PKT_DV              0x00000400
#define PKT_RDY             0x00000800

/*
 * USB_CTRL: USB Control Definitions
 */
#define UCTRL_RD_EN         0x00000001
#define UCTRL_WR_EN         0x00000002

/*
 * CMD_CODE: USB Command Codes
 */
#define CMD_SET_ADDR        0x00D00500
#define CMD_CFG_DEV         0x00D80500
#define CMD_SET_MODE        0x00F30500
#define CMD_RD_FRAME        0x00F50500
#define DAT_RD_FRAME        0x00F50200
#define CMD_RD_TEST         0x00FD0500
#define DAT_RD_TEST         0x00FD0200
#define CMD_SET_DEV_STAT    0x00FE0500
#define CMD_GET_DEV_STAT    0x00FE0500
#define DAT_GET_DEV_STAT    0x00FE0200
#define CMD_GET_ERR_CODE    0x00FF0500
#define DAT_GET_ERR_CODE    0x00FF0200
#define CMD_RD_ERR_STAT     0x00FB0500
#define DAT_RD_ERR_STAT     0x00FB0200
#define DAT_WR_BYTE(x)     (0x00000100 | ((x) << 16))
#define CMD_SEL_EP(x)      (0x00000500 | ((x) << 16))
#define DAT_SEL_EP(x)      (0x00000200 | ((x) << 16))
#define CMD_SEL_EP_CLRI(x) (0x00400500 | ((x) << 16))
#define DAT_SEL_EP_CLRI(x) (0x00400200 | ((x) << 16))
#define CMD_SET_EP_STAT(x) (0x00400500 | ((x) << 16))
#define CMD_CLR_BUF         0x00F20500
#define DAT_CLR_BUF         0x00F20200
#define CMD_VALID_BUF       0x00FA0500

/*
 * Device Status Register Definitions
 */
#define DSTAT_CONN          0x01
#define DSTAT_CONN_CH       0x02
#define DSTAT_SUSP          0x04
#define DSTAT_SUSP_CH       0x08
#define DSTAT_RST           0x10

/*
 * Endpoint Status Register Definitions
 */
#define EPSTAT_STALL        0x01
#define EPSTAT_DA           0x20
#define EPSTAT_RF_MO        0x40
#define EPSTAT_CND_ST       0x80


#endif  /* __USBREG_H__ */
